@@ -1,6 +1,6 @@
 package Gearman::Worker;
 use version;
-$Gearman::Worker::VERSION = version->declare("2.004.009");
+$Gearman::Worker::VERSION = version->declare("2.004.010");
 
 use strict;
 use warnings;
@@ -110,6 +110,12 @@ integers.
 See the L<Gearman::Client> documentation for a sample client sending the
 I<sum> job.
 
+=head1 NOTE
+
+If you intend using UTF-8 data with SSL based connection,
+beware there is no UTF-8 support in underlying L<Net::SSLeay>.
+L<perlunicode/"Forcing-Unicode-in-Perl-(Or-Unforcing-Unicode-in-Perl)"> describes proper workarounds.
+
 =head1 METHODS
 
 =cut
@@ -118,7 +124,6 @@ use Carp          ();
 use Gearman::Util ();
 use Gearman::Job;
 use Storable ();
-use IO::Select;
 
 use fields (
     'last_connect_fail',    # host:port -> unixtime
@@ -221,6 +226,9 @@ sub work {
     # in the loop we contact all servers.
     my %active_js = map { $_ => 1 } keys(%js_map);
 
+    # ( js => last_update_time, ... )
+    my %last_update_time;
+
     while (1) {
 
         # "Jobby" job servers are the set of server which we will contact
@@ -237,8 +245,11 @@ sub work {
             my $js_index = ($i + $js_offset) % $js_count;
             my $js_str   = $jobby_js[$js_index];
             my $js       = $js_map{$js_str};
-            my $jss      = $self->_get_js_sock($js, on_connect => $on_connect, register_on_reconnect => 1)
-                or next;
+            my $jss      = $self->_get_js_sock(
+                $js,
+                on_connect            => $on_connect,
+                register_on_reconnect => 1
+            ) or next;
 
             # TODO: add an optional sleep in here for the test suite
             # to test gearmand server going away here.  (SIGPIPE on
@@ -253,6 +264,7 @@ sub work {
                 } ## end if ($!{EPIPE} && $self...)
 
                 $self->_uncache_sock($js, "grab_job_timeout");
+                delete $last_update_time{$js_str};
                 next;
             } ## end unless (_send($jss, \$grab_req...))
 
@@ -264,6 +276,7 @@ sub work {
             unless (Gearman::Util::wait_for_readability($jss->fileno, $timeout))
             {
                 $self->_uncache_sock($js, "grab_job_timeout");
+                delete $last_update_time{$js_str};
                 next;
             } ## end unless (Gearman::Util::wait_for_readability...)
 
@@ -273,14 +286,17 @@ sub work {
                 $res = Gearman::Util::read_res_packet($jss, \$err);
                 unless ($res) {
                     $self->_uncache_sock($js, "read_res_error");
+                    delete $last_update_time{$js_str};
                     next;
                 }
             } while ($res->{type} eq "noop");
 
             if ($res->{type} eq "no_job") {
                 unless (_send($jss, \$presleep_req)) {
+                    delete $last_update_time{$js_str};
                     $self->_uncache_sock($js, "write_presleep_error");
                 }
+                $last_update_time{$js_str} = time;
                 next;
             } ## end if ($res->{type} eq "no_job")
 
@@ -313,8 +329,7 @@ sub work {
             my $err     = $@;
             warn "Job '$ability' died: $err" if $err;
 
-            $last_job_time = time();
-
+            $last_update_time{$js_str} = $last_job_time = time();
             if ($err) {
                 my $exception_req
                     = _rc("work_exception",
@@ -346,29 +361,54 @@ sub work {
                 $self->_uncache_sock($js, "write_res_error");
                 next;
             }
-        } ## end for (my $i = 0; $i < $js_count...)
-
-        foreach my $js_str (keys(%js_map)) {
-            my $jss
-                = $self->_get_js_sock($js_map{$js_str},
-                on_connect => $on_connect, register_on_reconnect => 1);
-              $jss || next;
-
-            my $io = IO::Select->new($jss);
-            $io->can_write(10) || next;
-
-            if($last_job_time && time() - $last_job_time >= 10) {
-                # chill for some arbitrary time until we're woken up again
-                select($io->bits(), undef, undef, 10 + rand(2));
-            }
 
             $active_js{$js_str} = 1;
+        } ## end for (my $i = 0; $i < $js_count...)
+
+        my @jss;
+
+        foreach my $js_str (keys(%js_map)) {
+            my $jss = $self->_get_js_sock(
+                $js_map{$js_str},
+                on_connect            => $on_connect,
+                register_on_reconnect => 1
+            ) or next;
+            push @jss, [$js_str, $jss];
         } ## end foreach my $js_str (keys(%js_map...))
 
+        my $wake_vec = '';
+
+        foreach my $j (@jss) {
+            (undef, my $_jss) = @{$j};
+            my $fd = $_jss->fileno;
+            vec($wake_vec, $fd, 1) = 1;
+        }
+
+        my $timeout = keys(%active_js) ? 0 : (10 + rand(2));
+
+        # chill for some arbitrary time until we're woken up again
+        my $nready = select(my $wout = $wake_vec, undef, undef, $timeout);
+
+        if ($nready) {
+            foreach my $j (@jss) {
+                my ($js_str, $jss) = @{$j};
+                my $fd = $jss->fileno;
+                $active_js{$js_str} = 1
+                    if vec($wout, $fd, 1);
+            } ## end foreach my $j (@jss)
+        } ## end if ($nready)
+
         my $is_idle = scalar(keys %active_js) > 0 ? 0 : 1;
 
-        last if $stop_if->($is_idle, $last_job_time);
+        return if $stop_if->($is_idle, $last_job_time);
+
+        my $update_since = time - (15 + rand 60);
+
+        while (my ($js_str, $last_update) = each %last_update_time) {
+            $active_js{$js_str} = 1 if $last_update < $update_since;
+        }
     } ## end while (1)
+
 } ## end sub work
 
 =head2 $worker->register_function($funcname, $subref)
@@ -625,7 +665,7 @@ sub _get_js_sock {
     delete $self->{last_connect_fail}{$js_str};
     delete $self->{down_since}{$js_str};
 
-    if( $opts{register_on_reconnect} ) {
+    if ($opts{register_on_reconnect}) {
         my @fail = ();
         foreach (keys %{ $self->{can} }) {
             $self->_register_function($_, $js, $sock) || push @fail, $_;
@@ -636,7 +676,7 @@ sub _get_js_sock {
                 @fail);
             return;
         }
-    } ## end if (delete $self->{down_since...})
+    } ## end if ($opts{register_on_reconnect...})
 
     $self->_sock_cache($js, $sock);
 
